package de.juplo.facebook.errors;


import de.juplo.facebook.errors.GraphApiException.Type;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;



/**
 *
 * @author Kai Moritz
 */
public class GraphApiExceptionTest
{
  private static final Logger LOG =
      LoggerFactory.getLogger(GraphApiExceptionTest.class);

  private HttpStatus status = HttpStatus.I_AM_A_TEAPOT;
  private HttpHeaders headers = HttpHeaders.EMPTY;


  @Test
  public void testError1()
  {
    String str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"An unknown error has occurred.\",\n" +
        "    \"type\": \"OAuthException\",\n" +
        "    \"code\": 1\n" +
        "  }\n" +
        "}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(UnknownErrorException.class));
    LOG.debug("{}", e.toString());
  }

  @Test
  public void testError2()
  {
    String str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"An unexpected error has occurred. Please retry your request later.\",\n" +
        "    \"type\": \"OAuthException\",\n" +
        "    \"code\": 2\n" +
        "  }\n" +
        "}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(UnexpectedErrorException.class));
    LOG.debug("{}", e.toString());
    assertEquals(new Integer(2), e.getCode());
    assertEquals("An unexpected error has occurred. Please retry your request later.", e.getMessage());
    assertEquals(Type.OAuthException, e.getType());
  }

  @Test
  public void testError4()
  {
    String str =
        "{\n" +
        "    \"error\": {\n" +
        "        \"code\": 4, \n" +
        "        \"fbtrace_id\": \"HZRM6BTMu+D\", \n" +
        "        \"is_transient\": true, \n" +
        "        \"message\": \"(#4) Application request limit reached\", \n" +
        "        \"type\": \"OAuthException\"\n" +
        "    }\n" +
        "}\n";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(ApplicationRequestLimitReachedException.class));
    LOG.debug("{}", e.toString());
    assertEquals(new Integer(4), e.getCode());
    assertEquals("(#4) Application request limit reached", e.getMessage());
    assertEquals(Type.OAuthException, e.getType());
  }

  @Test
  public void testError12()
  {
    String str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"(#12) location field is deprecated for versions v2.5 and higher\",\n" +
        "    \"type\": \"OAuthException\",\n" +
        "    \"code\": 12\n," +
        "    \"fbtrace_id\":\"BoxCYne7GrL\"\n" +
        "  }\n" +
        "}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(DeprecatedException.class));
    LOG.debug("{}", e.toString());
    assertEquals(new Integer(12), e.getCode());
    assertEquals("(#12) location field is deprecated for versions v2.5 and higher", e.getMessage());
    assertEquals(Type.OAuthException, e.getType());
  }

  @Test
  public void testError21()
  {
    String str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"(#21) Page ID 590408587650316 was migrated to page ID 1421620791415603.  Please update your API calls to the new ID\",\n" +
        "    \"type\": \"OAuthException\",\n" +
        "    \"code\": 21\n" +
        "  }\n" +
        "}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(PageMigratedException.class));
    LOG.debug("{}", e.toString());
    assertEquals(new Integer(21), e.getCode());
    assertEquals("(#21) Page ID 590408587650316 was migrated to page ID 1421620791415603.  Please update your API calls to the new ID", e.getMessage());
    assertEquals(Type.OAuthException, e.getType());
  }

  @Test
  public void testError100()
  {
    String str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"Unsupported get request.\",\n" +
        "    \"type\": \"GraphMethodException\",\n" +
        "    \"code\": 100\n" +
        "  }\n" +
        "}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);


    assertTrue(e.getClass().equals(UnsupportedGetRequestException.class));
    LOG.debug("{}", e.toString());
    assertEquals(new Integer(100), e.getCode());
    assertEquals("Unsupported get request.", e.getMessage());
    assertEquals(Type.GraphMethodException, e.getType());
  }

  @Test
  public void testError102()
  {
    String str ="{\"error\":{\"message\":\"A user access token is required to request this resource.\",\"type\":\"OAuthException\",\"code\":102,\"fbtrace_id\":\"DhdMyf23Ki7\"}}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);


    assertTrue(e.getClass().equals(UserAccessTokenRequiredException.class));
    LOG.debug("{}", e.toString());
  }

  @Test
  public void testError104()
  {
    String str ="{\"error\":{\"message\":\"An access token is required to request this resource.\",\"type\":\"OAuthException\",\"code\":104,\"fbtrace_id\":\"E2Jjkj5++LL\"}}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(AccessTokenRequiredException.class));
    LOG.debug("{}", e.toString());
  }

  @Test
  public void testError190()
  {
    LOG.info("testError190");

    String str ="{\"error\":{\"message\":\"Bad signature\",\"type\":\"OAuthException\",\"code\":190,\"fbtrace_id\":\"Ay2OYQrINbXOCfQpBvoxDIw\"}}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(AccessTokenExpiredException.class));
    LOG.debug("{}", e.toString());
  }

  @Test
  public void testError200()
  {
    String str ="{\n" +
        "  \"error\": {\n" +
        "    \"message\": \"(#200) The user hasn't authorized the application to perform this action\",\n" +
        "    \"type\": \"OAuthException\",\n" +
        "    \"code\": 200\n" +
        "  }\n" +
        "}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(ApplicationNotAuthorizedByUserException.class));
    LOG.debug("{}", e.toString());
  }

  @Test
  public void testError613()
  {
    String str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"(#613) Calls to stream have exceeded the rate of 600 calls per 600 seconds.\",\n" +
        "    \"type\": \"OAuthException\",\n" +
        "    \"code\": 613\n" +
        "  }\n" +
        "}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(RateLimitExceededException.class));
    LOG.debug("{}", e.toString());
    assertEquals(new Integer(613), e.getCode());
    assertEquals("(#613) Calls to stream have exceeded the rate of 600 calls per 600 seconds.", e.getMessage());
    assertEquals(Type.OAuthException, e.getType());
  }

  @Test
  public void testError2200()
  {
    String str ="{\"error\":{\"message\":\"(#2200) callback verification failed: \",\"type\":\"OAuthException\",\"code\":2200,\"fbtrace_id\":\"ESLjoZKvPXg\"}}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(CallbackVerificationFailedException.class));
    LOG.debug("{}", e.toString());
  }

  @Test
  public void testError2500()
  {
    String str ="{\"error\":{\"message\":\"An active access token must be used to query information about the current user.\",\"type\":\"OAuthException\",\"code\":2500,\"fbtrace_id\":\"AUmMHJlVYvTbQrBT11Dk104\"}}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(ActiveAccessTokenMissingException.class));
    LOG.debug("{}", e.toString());
  }

  @Test
  public void testUnmappedError()
  {
    String str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"This error does not exist.\",\n" +
        "    \"type\": \"NonexistentTypeException\",\n" +
        "    \"code\": 999999999\n" +
        "  }\n" +
        "}";
    byte[] message = str.getBytes();
    GraphApiException e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(UnmappedErrorException.class));
    LOG.debug("{}", e.toString());
    assertEquals(new Integer(999999999), e.getCode());
    assertEquals("This error does not exist.", e.getMessage());
    try
    {
      Type type = e.getType();
      LOG.error("unknown type: {}", type);
      fail("unmapped type was resolved by enum: " + type);
    }
    catch (IllegalArgumentException ee) {}
  }

  @Test
  public void testUnmappedErrors()
  {
    String str;
    byte[] message;
    GraphApiException e;


    str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": null,\n" +
        "    \"type\": \"WhateverTypeException\",\n" +
        "    \"code\": 999999999\n" +
        "  }\n" +
        "}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(UnmappedErrorException.class));
    LOG.debug("{}", e.toString());
    assertNull(e.getMessage());
    try
    {
      Type type = e.getType();
      LOG.error("unknown type: {}", type);
      fail("unmapped type was resolved by enum: " + type);
    }
    catch (IllegalArgumentException ee) {}
    assertEquals(new Integer(999999999), e.getCode());
    assertNull(e.getSubCode());
    assertNull(e.getUserTitle());
    assertNull(e.getUserMessage());
    assertNull(e.getTraceId());


    str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"type\": \"WhateverTypeException\",\n" +
        "    \"code\": 999999999\n" +
        "  }\n" +
        "}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(UnmappedErrorException.class));
    LOG.debug("{}", e.toString());
    assertNull(e.getMessage());
    try
    {
      Type type = e.getType();
      LOG.error("unknown type: {}", type);
      fail("unmapped type was resolved by enum: " + type);
    }
    catch (IllegalArgumentException ee) {}
    assertEquals(new Integer(999999999), e.getCode());
    assertNull(e.getSubCode());
    assertNull(e.getUserTitle());
    assertNull(e.getUserMessage());
    assertNull(e.getTraceId());


    str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"An unmapped Graph-API-Exception.\",\n" +
        "    \"type\": null,\n" +
        "    \"code\": 999999999\n" +
        "  }\n" +
        "}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(UnmappedErrorException.class));
    LOG.debug("{}", e.toString());
    assertEquals("An unmapped Graph-API-Exception.", e.getMessage());
    assertNull(e.getType());
    assertEquals(new Integer(999999999), e.getCode());
    assertNull(e.getSubCode());
    assertNull(e.getUserTitle());
    assertNull(e.getUserMessage());
    assertNull(e.getTraceId());


    str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"An unmapped Graph-API-Exception.\",\n" +
        "    \"code\": 999999999\n" +
        "  }\n" +
        "}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);

    assertTrue(e.getClass().equals(UnmappedErrorException.class));
    LOG.debug("{}", e.toString());
    assertEquals("An unmapped Graph-API-Exception.", e.getMessage());
    assertNull(e.getType());
    assertEquals(new Integer(999999999), e.getCode());
    assertNull(e.getSubCode());
    assertNull(e.getUserTitle());
    assertNull(e.getUserMessage());
    assertNull(e.getTraceId());
  }

  @Test
  public void testInvlalidErrors()
  {
    String str;
    byte[] message;
    GraphApiException e;


    str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"Not a Graph-Api-Exception.\",\n" +
        "    \"type\": \"Whatever\",\n" +
        "    \"code\": \"some string\"\n" +
        "  }\n" +
        "}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"Not a Graph-Api-Exception.\",\n" +
        "    \"type\": \"Whatever\",\n" +
        "    \"code\": 9.9\n" +
        "  }\n" +
        "}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"Not a Graph-Api-Exception.\",\n" +
        "    \"type\": \"Whatever\",\n" +
        "    \"code\": null\n" +
        "  }\n" +
        "}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str =
        "{\n" +
        "  \"error\":\n" +
        "  {\n" +
        "    \"message\": \"Not a Graph-Api-Exception.\",\n" +
        "    \"type\": \"Whatever\"\n" +
        "  }\n" +
        "}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="{\"error\":{\"message\":null}}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="{\"error\":{\"type\":null}}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="{\"error\":{\"code\":null}}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="{\"error\":{}}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="{\"error\":\"some message\"}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="{\"error\":null}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="{\"some filed\":\"some message\"}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="{}";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());


    str ="";
    message = str.getBytes();
    e = GraphApiException.create (status, headers, message);
    assertTrue(e.getClass().equals(ErrorResponseParsingErrorException.class));
    LOG.debug("{}", e.toString());
  }
}
